@@ -94,10 +94,7 @@ class MessageCode(Enum):
 BOARDS = [
     Boards.STM32F407G_DISCO.value,
     Boards.STM32F411E_DISCO.value,
-    Boards.STM32F469I_DISCO.value,
     Boards.STM32F769I_DISCO.value,
-    Boards.STM32H735G_DK.value,
-    Boards.STM32H747I_DISCO.value,
     Boards.IMXRT1050_EVKB.value,
 ]
 
@@ -116,12 +113,9 @@ LANGUAGE_TO_CODE = dict(
 )
 
 AUDIO_DEBUG_RECORD_DURATION_SEC = {
-    Boards.STM32F469I_DISCO.value: 3,
     Boards.STM32F769I_DISCO.value: 3,
-    Boards.STM32H747I_DISCO.value: 3,
     Boards.STM32F407G_DISCO.value: 1,
     Boards.STM32F411E_DISCO.value: 1,
-    Boards.STM32H735G_DK.value: 3,
     Boards.IMXRT1050_EVKB.value: 3
 }
 
@@ -135,6 +129,8 @@ MONITORING_CENTER_SMOOTHING_FACTOR_CPU_USAGE = 0.3
 
 SERIAL_CONNECTION_RETRIES = 10
 
+NUM_MODEL_NAME_SECTIONS = 6
+
 NO_UPLOAD = False
 DEBUG = False
 
@@ -406,6 +402,10 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
         self._upload_button.state(["disabled"])
         self._boards_tree_view.state(["disabled"])
         self._boards_tree_view.unbind('<ButtonRelease-1>')
+        self._boards_tree_view.config(selectmode="none")
+        self._languages_tree_view.state(["disabled"])
+        self._languages_tree_view.unbind('<ButtonRelease-1>')
+        self._languages_tree_view.config(selectmode="none")
         try:
             if not NO_UPLOAD:
                 upload_firmware(Boards(self._board), LANGUAGE_TO_CODE[self._language])
@@ -416,6 +416,10 @@ class UploadFirmwareFrame(tk.Frame, PicoFrame):
             self._upload_button.state(["!disabled"])
             self._boards_tree_view.state(["!disabled"])
             self._boards_tree_view.bind('<ButtonRelease-1>', self._update_selected_board)
+            self._boards_tree_view.config(selectmode="browse")
+            self._languages_tree_view.state(["!disabled"])
+            self._languages_tree_view.bind('<ButtonRelease-1>', self._update_selected_language)
+            self._languages_tree_view.config(selectmode="browse")
             self._uploading_label.stop_loading()
             self._is_status_update_done = True
         else:
@@ -646,7 +650,7 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
 
     def _connect(self):
         for _ in range(SERIAL_CONNECTION_RETRIES):
-            for port in serial.tools.list_ports.comports():
+            for port in comports():
                 try:
                     self._port = port
                     self._serial_driver = serial.Serial(port=port.device, baudrate=115200)
@@ -756,24 +760,41 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
                 self._select_rhino_model_button.state(["!disabled"])
             else:
                 self._is_uploading_done = True
-                
+
         self._is_status_update_done = True
 
+    @staticmethod
+    def _is_model_name_valid(model_name, language):
+        return '_cortexm_' in model_name \
+            and ('_%s_' % LANGUAGE_TO_CODE[language]) in model_name \
+            and len(model_name.split('_')) == NUM_MODEL_NAME_SECTIONS
+
+    @staticmethod
+    def _model_name_parser(model_name):
+        title_and_language = model_name.split('_cortexm')[0].split('_')
+        polished_title_and_language = ' '.join(title_and_language[0:-1]).replace('-', ' ') \
+                                      + ' [' + title_and_language[-1] + ']'
+        polished_title_and_language = ' '.join([x.capitalize() for x in polished_title_and_language.split()])
+        return polished_title_and_language
+
     def _select_porcupine_model(self):
+        previous_porcupine_model_path = self._porcupine_model_path
         self._porcupine_model_path = filedialog.askopenfilename(
             initialdir=os.path.dirname(os.path.expanduser('~/')),
             title="Select Porcupine Wake Word Model (.PPN)",
             filetypes=(("ppn files", "*.ppn"), ("all files", "*.*")))
-
         if (type(self._porcupine_model_path) is not tuple) and self._porcupine_model_path:
-            if '_cortexm_' not in os.path.basename(self._porcupine_model_path):
+            porcupine_model_name = os.path.basename(self._porcupine_model_path)
+            if self._is_model_name_valid(porcupine_model_name, self._language):
+                porcupine_model_name = self._model_name_parser(porcupine_model_name)
+            else:
                 messagebox.showwarning(
                     title='warning',
                     message='It seems that the selected model belongs to a '
-                            'different platform.\n Please make sure that the '
+                            'different platform\\language.\n '
+                            'Please make sure that the '
                             'right model is chosen before proceeding.')
-            porcupine_model_name = os.path.basename(self._porcupine_model_path).split('_cortexm')[0].replace('_', ' ')
-            porcupine_model_name = ' '.join([x.capitalize() for x in porcupine_model_name.split()])
+
             self._selected_models['porcupine'] = porcupine_model_name
             self._porcupine_selected_model_entry.state(["!disabled"])
             self._porcupine_selected_model_entry.delete(0, tk.END)
@@ -781,27 +802,28 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             self._porcupine_selected_model_entry.state(["disabled"])
             self._upload_models_button.config(text='Upload ⇨', command=self._upload_models)
         else:
-            self._porcupine_model_path = None
+            self._porcupine_model_path = previous_porcupine_model_path
             self._porcupine_selected_model_entry.insert(0, '')
             self._upload_models_button.state(["!disabled"])
 
     def _select_rhino_model(self):
+        previous_rhino_model_path = self._rhino_model_path
         self._rhino_model_path = filedialog.askopenfilename(
             initialdir=os.path.dirname(os.path.expanduser('~/')),
             title="Select Rhino Speech-to-Intent Model (.RHN)",
             filetypes=(("rhn files", "*.rhn"), ("all files", "*.*")))
         if (type(self._rhino_model_path) is not tuple) and self._rhino_model_path:
-            if '_cortexm_' not in os.path.basename(self._rhino_model_path) or \
-                    ('_%s_' % LANGUAGE_TO_CODE[self._language]) not in os.path.basename(self._rhino_model_path):
+            rhino_model_name = os.path.basename(self._rhino_model_path)
+            if self._is_model_name_valid(rhino_model_name, self._language):
+                rhino_model_name = self._model_name_parser(rhino_model_name)
+            else:
                 messagebox.showwarning(
                     title='warning',
                     message='It seems that the selected model belongs to a '
-                            'different platform\language.\n '
+                            'different platform\\language.\n '
                             'Please make sure that the '
                             'right model is chosen before proceeding.')
-            raw_name = os.path.basename(self._rhino_model_path).split('_cortexm')[0].split('_')
-            rhino_model_name = ' '.join(raw_name[0:-1]) + ' [' + raw_name[-1] + ']'
-            rhino_model_name = ' '.join([x.capitalize() for x in rhino_model_name.split()])
+
             self._selected_models['rhino'] = rhino_model_name
             self._rhino_selected_model_entry.state(["!disabled"])
             self._rhino_selected_model_entry.delete(0, tk.END)
@@ -809,7 +831,7 @@ class UploadModelsFrame(tk.Frame, PicoFrame):
             self._rhino_selected_model_entry.state(["disabled"])
             self._upload_models_button.config(text='Upload ⇨', command=self._upload_models)
         else:
-            self._rhino_model_path = None
+            self._rhino_model_path = previous_rhino_model_path
             self._rhino_selected_model_entry.insert(0, '')
             self._upload_models_button.state(["!disabled"])
 
@@ -913,7 +935,7 @@ class ControlPanel(tk.Frame, PicoFrame):
 
     def _connect(self):
         for _ in range(SERIAL_CONNECTION_RETRIES):
-            for port in serial.tools.list_ports.comports():
+            for port in comports():
                 try:
                     self._port = port
                     self._serial_driver = serial.Serial(port=port.device, baudrate=115200)
